from datetime import datetime, timedelta

from django import forms
from django.conf import settings
from django.contrib.admin.widgets import AdminDateWidget
from django.core.exceptions import ValidationError
from django.utils import timezone
from django.utils.text import slugify
from django.utils.translation import gettext
from django.utils.translation import gettext_lazy as _

from jsonschema import validate

from ..constants import TRANSFORM_SCHEMA
from ..models import FormSubmission
from .exporter import Exporter
from .utils import PrettyJsonEncoder


def form_choices(modelClass):
    form_names = modelClass.objects.values_list('name', flat=True).distinct()

    for name in form_names.order_by('name'):
        yield (name, name)


def form_field_choices(fields):
    for field in fields:
        yield (field.field_id, field.label)


class BaseFormExportForm(forms.Form):
    excel_limit = 65536
    export_filename = 'export-{language}-{form_name}-%Y-%m-%d'

    form_name = forms.ChoiceField(choices=[])
    from_date = forms.DateField(
        label=_('from date'),
        required=False,
        widget=AdminDateWidget
    )
    to_date = forms.DateField(
        label=_('to date'),
        required=False,
        widget=AdminDateWidget
    )
    language = forms.ChoiceField(
        label=_('language'),
        choices=settings.LANGUAGES
    )

    def __init__(self, *args, **kwargs):
        super(BaseFormExportForm, self).__init__(*args, **kwargs)
        self.fields['form_name'].choices = form_choices(modelClass=self.model)

    def clean(self):
        if self.errors:
            return self.cleaned_data

        queryset = self.get_queryset()

        if queryset.count() >= self.excel_limit:
            error_message = _("Export failed! More than 65,536 entries found, exceeded Excel limitation!")
            raise forms.ValidationError(error_message)

        return self.cleaned_data

    def get_filename(self, extension=None):
        data = self.cleaned_data
        form_name = data['form_name'].lower()
        filename_format = self.export_filename.format(
            form_name=slugify(form_name),
            language=data['language'],
        )
        filename = timezone.now().strftime(filename_format)

        if extension:
            filename = '{}.{}'.format(filename, extension)
        return filename

    def get_queryset(self):
        data = self.cleaned_data
        from_date, to_date = data.get('from_date'), data.get('to_date')

        queryset = self.model.objects.filter(
            name=data['form_name'],
            language=data['language'],
        )

        if from_date:
            lower = datetime(*from_date.timetuple()[:6])  # inclusive
            queryset = queryset.filter(sent_at__gte=lower)

        if to_date:
            upper = datetime(*to_date.timetuple()[:6]) + timedelta(days=1)  # exclusive
            queryset = queryset.filter(sent_at__lt=upper)

        return queryset


class FormSubmissionExportForm(BaseFormExportForm):
    model = FormSubmission


class FormExportStep1Form(BaseFormExportForm):
    model = FormSubmission


class FormExportStep2Form(forms.Form):
    current_fields = forms.MultipleChoiceField(required=False)
    old_fields = forms.MultipleChoiceField(required=False)

    def __init__(self, *args, **kwargs):
        submissions = kwargs.pop('submissions')
        super(FormExportStep2Form, self).__init__(*args, **kwargs)

        exporter = Exporter(queryset=submissions)
        current_fields, old_fields = exporter.get_fields_for_export()

        pre_selected_fields = (field.field_id for field in current_fields)

        self.fields['current_fields'].choices = form_field_choices(current_fields)
        self.fields['current_fields'].initial = pre_selected_fields
        self.fields['old_fields'].choices = form_field_choices(old_fields)

    def get_fields(self):
        data = self.cleaned_data
        return data['current_fields'] + data['old_fields']

    def clean(self):
        if self.errors:
            return self.cleaned_data

        fields = self.get_fields()

        if not fields:
            message = gettext('Please select at least one field to export.')
            raise forms.ValidationError(message)
        return self.cleaned_data


class WebhookAdminForm(forms.ModelForm):

    transform = forms.JSONField(encoder=PrettyJsonEncoder, required=False)

    def clean_transform(self):
        data = self.cleaned_data["transform"]
        if data is not None:
            try:
                validate(instance=data, schema=TRANSFORM_SCHEMA)
            except Exception as err:
                raise ValidationError(err.message)
        return data



# Generated by Django 1.10.8 on 2017-12-20 19:33
from __future__ import unicode_literals

from django.db import migrations, models

import cms.models.fields

import djangocms_attributes_field.fields

from ..models import BaseFormPlugin


def forward_migration(apps, schema_editor):
    db_alias = schema_editor.connection.alias
    FieldPlugin = apps.get_model('aldryn_forms', 'FieldPlugin')
    for field in FieldPlugin.objects.using(db_alias).iterator():
        for idx, option in enumerate(field.option_set.order_by('value'), start=1):
            option.position = idx * 10
            option.save()


def backward_migration(apps, schema_editor):
    db_alias = schema_editor.connection.alias
    Option = apps.get_model('aldryn_forms', 'Option')
    Option.objects.using(db_alias).all().update(position=None)


class Migration(migrations.Migration):

    dependencies = [
        ('aldryn_forms', '0009_auto_20171218_1049'),
        ('cms', '0006_auto_20140924_1110'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='option',
            options={'verbose_name': 'Option', 'verbose_name_plural': 'Options'},
        ),
        migrations.AlterField(
            model_name='emailfieldplugin',
            name='placeholder_text',
            field=models.CharField(blank=True,
                                   help_text='Default text in a form. Disappears when user starts typing. Example: "email@example.com"',
                                   max_length=255, verbose_name='Placeholder text'),
        ),
        migrations.AlterField(
            model_name='fieldplugin',
            name='placeholder_text',
            field=models.CharField(blank=True,
                                   help_text='Default text in a form. Disappears when user starts typing. Example: "email@example.com"',
                                   max_length=255, verbose_name='Placeholder text'),
        ),
        migrations.AlterField(
            model_name='fileuploadfieldplugin',
            name='placeholder_text',
            field=models.CharField(blank=True,
                                   help_text='Default text in a form. Disappears when user starts typing. Example: "email@example.com"',
                                   max_length=255, verbose_name='Placeholder text'),
        ),
        migrations.AlterField(
            model_name='formplugin',
            name='form_template',
            field=models.CharField(choices=BaseFormPlugin.FORM_TEMPLATES, default=BaseFormPlugin.DEFAULT_FORM_TEMPLATE,
                                   max_length=255, verbose_name='form template'),
        ),
        migrations.AlterField(
            model_name='imageuploadfieldplugin',
            name='placeholder_text',
            field=models.CharField(blank=True,
                                   help_text='Default text in a form. Disappears when user starts typing. Example: "email@example.com"',
                                   max_length=255, verbose_name='Placeholder text'),
        ),
        migrations.AlterField(
            model_name='textareafieldplugin',
            name='placeholder_text',
            field=models.CharField(blank=True,
                                   help_text='Default text in a form. Disappears when user starts typing. Example: "email@example.com"',
                                   max_length=255, verbose_name='Placeholder text'),
        ),
        migrations.AlterModelOptions(
            name='option',
            options={'ordering': ('position',), 'verbose_name': 'Option', 'verbose_name_plural': 'Options'},
        ),
        migrations.RenameField(
            model_name='formplugin',
            old_name='page',
            new_name='redirect_page',
        ),
        migrations.AlterField(
            model_name='formplugin',
            name='redirect_page',
            field=cms.models.fields.PageField(blank=True, null=True, on_delete=models.deletion.SET_NULL,
                                              to='cms.Page', verbose_name='CMS Page'),
        ),
        migrations.AddField(
            model_name='emailfieldplugin',
            name='attributes',
            field=djangocms_attributes_field.fields.AttributesField(blank=True, default=dict, verbose_name='Attributes'),
        ),
        migrations.AddField(
            model_name='emailfieldplugin',
            name='initial_value',
            field=models.CharField(blank=True, help_text='Default value of field.', max_length=255, verbose_name='Initial value'),
        ),
        migrations.AddField(
            model_name='emailfieldplugin',
            name='name',
            field=models.CharField(blank=True, help_text='Used to set the field name', max_length=255, verbose_name='Name'),
        ),
        migrations.AddField(
            model_name='fieldplugin',
            name='attributes',
            field=djangocms_attributes_field.fields.AttributesField(blank=True, default=dict, verbose_name='Attributes'),
        ),
        migrations.AddField(
            model_name='fieldplugin',
            name='initial_value',
            field=models.CharField(blank=True, help_text='Default value of field.', max_length=255, verbose_name='Initial value'),
        ),
        migrations.AddField(
            model_name='fieldplugin',
            name='name',
            field=models.CharField(blank=True, help_text='Used to set the field name', max_length=255, verbose_name='Name'),
        ),
        migrations.AddField(
            model_name='fileuploadfieldplugin',
            name='attributes',
            field=djangocms_attributes_field.fields.AttributesField(blank=True, default=dict, verbose_name='Attributes'),
        ),
        migrations.AddField(
            model_name='fileuploadfieldplugin',
            name='initial_value',
            field=models.CharField(blank=True, help_text='Default value of field.', max_length=255, verbose_name='Initial value'),
        ),
        migrations.AddField(
            model_name='fileuploadfieldplugin',
            name='name',
            field=models.CharField(blank=True, help_text='Used to set the field name', max_length=255, verbose_name='Name'),
        ),
        migrations.AddField(
            model_name='formplugin',
            name='form_attributes',
            field=djangocms_attributes_field.fields.AttributesField(blank=True, default=dict, verbose_name='Attributes'),
        ),
        migrations.AddField(
            model_name='imageuploadfieldplugin',
            name='attributes',
            field=djangocms_attributes_field.fields.AttributesField(blank=True, default=dict, verbose_name='Attributes'),
        ),
        migrations.AddField(
            model_name='imageuploadfieldplugin',
            name='initial_value',
            field=models.CharField(blank=True, help_text='Default value of field.', max_length=255, verbose_name='Initial value'),
        ),
        migrations.AddField(
            model_name='imageuploadfieldplugin',
            name='name',
            field=models.CharField(blank=True, help_text='Used to set the field name', max_length=255, verbose_name='Name'),
        ),
        migrations.AddField(
            model_name='textareafieldplugin',
            name='attributes',
            field=djangocms_attributes_field.fields.AttributesField(blank=True, default=dict, verbose_name='Attributes'),
        ),
        migrations.AddField(
            model_name='textareafieldplugin',
            name='initial_value',
            field=models.CharField(blank=True, help_text='Default value of field.', max_length=255, verbose_name='Initial value'),
        ),
        migrations.AddField(
            model_name='textareafieldplugin',
            name='name',
            field=models.CharField(blank=True, help_text='Used to set the field name', max_length=255, verbose_name='Name'),
        ),
        migrations.AlterField(
            model_name='formplugin',
            name='redirect_type',
            field=models.CharField(blank=True, choices=[('redirect_to_page', 'CMS Page'), ('redirect_to_url', 'Absolute URL')], help_text='Where to redirect the user when the form has been successfully sent?', max_length=20, verbose_name='Redirect to'),
        ),

        # 1: create field as nullable
        migrations.AddField(
            model_name='option',
            name='position',
            field=models.PositiveIntegerField(blank=True, null=True, verbose_name='Position'),
        ),

        # 2: populate field
        migrations.RunPython(forward_migration, backward_migration),

        # 3: update field to non-nullable
        migrations.AlterField(
            model_name='option',
            name='position',
            field=models.PositiveIntegerField(blank=True, verbose_name='Position'),
        ),
    ]



from __future__ import unicode_literals

from django.contrib.auth.models import Permission
from django.contrib.contenttypes.models import ContentType
from django.db import migrations, models


APP_LABEL = 'email_notifications'
MODEL_NAME = 'emailnotificationformplugin'
VERBOSE_MODEL_NAME = 'form (advanced)'
ACTIONS = ('add', 'change', 'delete')


def get_content_type():
    try:
        return ContentType.objects.get(app_label=APP_LABEL, model=MODEL_NAME)
    except ContentType.DoesNotExist:
        return None


def forwards(apps, schema_editor):
    content_type = get_content_type()
    if content_type:
        for action in ACTIONS:
            Permission.objects.get_or_create(
                codename='{}_{}'.format(action, MODEL_NAME),
                content_type=content_type,
                defaults={
                    'name': 'Can {} {}'.format(action, VERBOSE_MODEL_NAME)
                }
            )


def backwards(apps, schema_editor):
    content_type = get_content_type()
    if content_type:
        codenames = [
            '{}_{}'.format(action, MODEL_NAME)
            for action in ACTIONS
        ]

        Permission.objects.filter(
            codename__in=codenames,
            content_type=content_type,
        ).delete()


class Migration(migrations.Migration):

    dependencies = [
        ('email_notifications', '0001_initial'),
    ]

    operations = [
        migrations.RunPython(
            forwards,
            reverse_code=backwards,
        )
    ]



from __future__ import unicode_literals

from django.conf import settings
from django.db import migrations, models

import cms.models.fields

import filer.fields.folder

import aldryn_forms.sizefield.models


class Migration(migrations.Migration):

    dependencies = [
        ('filer', '0002_auto_20150606_2003'),
        ('cms', '0003_auto_20140926_2347'),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='EmailFieldPlugin',
            fields=[
                ('cmsplugin_ptr', models.OneToOneField(parent_link=True, auto_created=True, primary_key=True, serialize=False, to='cms.CMSPlugin', on_delete=models.CASCADE)),
                ('label', models.CharField(max_length=50, verbose_name='Label', blank=True)),
                ('required', models.BooleanField(default=True, verbose_name='Field is required')),
                ('required_message', models.TextField(help_text='Error message displayed if the required field is left empty. Default: "This field is required".', null=True, verbose_name='Error message', blank=True)),
                ('placeholder_text', models.CharField(help_text='Default text in a form. Disappears when user starts typing. Example: "email@exmaple.com"', max_length=50, verbose_name='Placeholder text', blank=True)),
                ('help_text', models.TextField(help_text='Explanatory text displayed next to input field. Just like this one.', null=True, verbose_name='Help text', blank=True)),
                ('min_value', models.PositiveIntegerField(null=True, verbose_name='Min value', blank=True)),
                ('max_value', models.PositiveIntegerField(null=True, verbose_name='Max value', blank=True)),
                ('custom_classes', models.CharField(max_length=200, verbose_name='custom css classes', blank=True)),
                ('email_send_notification', models.BooleanField(default=False, help_text='When checked, the value of this field will be used to send an email notification.', verbose_name='send notification when form is submitted')),
                ('email_subject', models.CharField(default='', help_text='Used as the email subject when email_send_notification is checked.', max_length=200, verbose_name='email subject', blank=True)),
                ('email_body', models.TextField(default='', help_text='Additional body text used when email notifications are active.', verbose_name='Additional email body', blank=True)),
            ],
            options={
                'abstract': False,
            },
            bases=('cms.cmsplugin',),
        ),
        migrations.CreateModel(
            name='FieldPlugin',
            fields=[
                ('cmsplugin_ptr', models.OneToOneField(parent_link=True, auto_created=True, primary_key=True, serialize=False, to='cms.CMSPlugin', on_delete=models.CASCADE)),
                ('label', models.CharField(max_length=50, verbose_name='Label', blank=True)),
                ('required', models.BooleanField(default=True, verbose_name='Field is required')),
                ('required_message', models.TextField(help_text='Error message displayed if the required field is left empty. Default: "This field is required".', null=True, verbose_name='Error message', blank=True)),
                ('placeholder_text', models.CharField(help_text='Default text in a form. Disappears when user starts typing. Example: "email@exmaple.com"', max_length=50, verbose_name='Placeholder text', blank=True)),
                ('help_text', models.TextField(help_text='Explanatory text displayed next to input field. Just like this one.', null=True, verbose_name='Help text', blank=True)),
                ('min_value', models.PositiveIntegerField(null=True, verbose_name='Min value', blank=True)),
                ('max_value', models.PositiveIntegerField(null=True, verbose_name='Max value', blank=True)),
                ('custom_classes', models.CharField(max_length=200, verbose_name='custom css classes', blank=True)),
            ],
            options={
                'abstract': False,
            },
            bases=('cms.cmsplugin',),
        ),
        migrations.CreateModel(
            name='FieldsetPlugin',
            fields=[
                ('cmsplugin_ptr', models.OneToOneField(parent_link=True, auto_created=True, primary_key=True, serialize=False, to='cms.CMSPlugin', on_delete=models.CASCADE)),
                ('legend', models.CharField(max_length=50, verbose_name='Legend', blank=True)),
                ('custom_classes', models.CharField(max_length=200, verbose_name='custom css classes', blank=True)),
            ],
            options={
                'abstract': False,
            },
            bases=('cms.cmsplugin',),
        ),
        migrations.CreateModel(
            name='FileUploadFieldPlugin',
            fields=[
                ('cmsplugin_ptr', models.OneToOneField(parent_link=True, auto_created=True, primary_key=True, serialize=False, to='cms.CMSPlugin', on_delete=models.CASCADE)),
                ('label', models.CharField(max_length=50, verbose_name='Label', blank=True)),
                ('required', models.BooleanField(default=True, verbose_name='Field is required')),
                ('required_message', models.TextField(help_text='Error message displayed if the required field is left empty. Default: "This field is required".', null=True, verbose_name='Error message', blank=True)),
                ('placeholder_text', models.CharField(help_text='Default text in a form. Disappears when user starts typing. Example: "email@exmaple.com"', max_length=50, verbose_name='Placeholder text', blank=True)),
                ('help_text', models.TextField(help_text='Explanatory text displayed next to input field. Just like this one.', null=True, verbose_name='Help text', blank=True)),
                ('min_value', models.PositiveIntegerField(null=True, verbose_name='Min value', blank=True)),
                ('max_value', models.PositiveIntegerField(null=True, verbose_name='Max value', blank=True)),
                ('custom_classes', models.CharField(max_length=200, verbose_name='custom css classes', blank=True)),
                ('max_size', aldryn_forms.sizefield.models.FileSizeField(help_text='The maximum file size of the upload, in bytes. You can use common size suffixes (kB, MB, GB, ...).', null=True, verbose_name='Maximum file size', blank=True)),
                ('upload_to', filer.fields.folder.FilerFolderField(verbose_name='Upload files to', to='filer.Folder', help_text='Select a folder to which all files submitted through this field will be uploaded to.', on_delete=models.CASCADE)),
            ],
            options={
                'abstract': False,
            },
            bases=('cms.cmsplugin',),
        ),
        migrations.CreateModel(
            name='FormButtonPlugin',
            fields=[
                ('cmsplugin_ptr', models.OneToOneField(parent_link=True, auto_created=True, primary_key=True, serialize=False, to='cms.CMSPlugin', on_delete=models.CASCADE)),
                ('label', models.CharField(max_length=50, verbose_name='Label')),
                ('custom_classes', models.CharField(max_length=200, verbose_name='custom css classes', blank=True)),
            ],
            options={
                'abstract': False,
            },
            bases=('cms.cmsplugin',),
        ),
        migrations.CreateModel(
            name='FormData',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('name', models.CharField(max_length=50, editable=False, db_index=True)),
                ('data', models.TextField(null=True, editable=False, blank=True)),
                ('language', models.CharField(default='en', max_length=10, verbose_name='language', choices=[('de', 'German'), ('en', 'English'), ('fr', 'French')])),
                ('people_notified', models.TextField(help_text='People who got a notification when form was submitted.', verbose_name='users notified', editable=False, blank=True)),
                ('sent_at', models.DateTimeField(auto_now_add=True)),
            ],
            options={
                'verbose_name': 'Form submission',
                'verbose_name_plural': 'Form submissions',
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='FormPlugin',
            fields=[
                ('cmsplugin_ptr', models.OneToOneField(parent_link=True, auto_created=True, primary_key=True, serialize=False, to='cms.CMSPlugin', on_delete=models.CASCADE)),
                ('name', models.CharField(help_text='Used to filter out form submissions.', max_length=50, verbose_name='Name')),
                ('error_message', models.TextField(help_text="An error message that will be displayed if the form doesn't validate.", null=True, verbose_name='Error message', blank=True)),
                ('success_message', models.TextField(help_text='An success message that will be displayed.', null=True, verbose_name='Success message', blank=True)),
                ('redirect_type', models.CharField(help_text='Where to redirect the user when the form has been successfully sent?', max_length=20, verbose_name='Redirect to', choices=[('redirect_to_page', 'CMS Page'), ('redirect_to_url', 'Absolute URL')])),
                ('url', models.URLField(null=True, verbose_name='Absolute URL', blank=True)),
                ('custom_classes', models.CharField(max_length=200, verbose_name='custom css classes', blank=True)),
                ('form_template', models.CharField(default='aldryn_forms/form.html', max_length=200, verbose_name='form template', choices=[('aldryn_forms/form.html', 'Default')])),
                ('page', cms.models.fields.PageField(verbose_name='CMS Page', blank=True, to='cms.Page', null=True)),
                ('recipients', models.ManyToManyField(help_text='People who will get the form content via e-mail.', to=settings.AUTH_USER_MODEL, verbose_name='Recipients', blank=True)),
            ],
            options={
                'abstract': False,
            },
            bases=('cms.cmsplugin',),
        ),
        migrations.CreateModel(
            name='ImageUploadFieldPlugin',
            fields=[
                ('cmsplugin_ptr', models.OneToOneField(parent_link=True, auto_created=True, primary_key=True, serialize=False, to='cms.CMSPlugin', on_delete=models.CASCADE)),
                ('label', models.CharField(max_length=50, verbose_name='Label', blank=True)),
                ('required', models.BooleanField(default=True, verbose_name='Field is required')),
                ('required_message', models.TextField(help_text='Error message displayed if the required field is left empty. Default: "This field is required".', null=True, verbose_name='Error message', blank=True)),
                ('placeholder_text', models.CharField(help_text='Default text in a form. Disappears when user starts typing. Example: "email@exmaple.com"', max_length=50, verbose_name='Placeholder text', blank=True)),
                ('help_text', models.TextField(help_text='Explanatory text displayed next to input field. Just like this one.', null=True, verbose_name='Help text', blank=True)),
                ('min_value', models.PositiveIntegerField(null=True, verbose_name='Min value', blank=True)),
                ('max_value', models.PositiveIntegerField(null=True, verbose_name='Max value', blank=True)),
                ('custom_classes', models.CharField(max_length=200, verbose_name='custom css classes', blank=True)),
                ('max_size', aldryn_forms.sizefield.models.FileSizeField(help_text='The maximum file size of the upload, in bytes. You can use common size suffixes (kB, MB, GB, ...).', null=True, verbose_name='Maximum file size', blank=True)),
                ('max_width', models.PositiveIntegerField(help_text='The maximum width of the uploaded image, in pixels.', null=True, verbose_name='Maximum image width', blank=True)),
                ('max_height', models.PositiveIntegerField(help_text='The maximum height of the uploaded image, in pixels.', null=True, verbose_name='Maximum image height', blank=True)),
                ('upload_to', filer.fields.folder.FilerFolderField(verbose_name='Upload files to', to='filer.Folder', help_text='Select a folder to which all files submitted through this field will be uploaded to.', on_delete=models.CASCADE)),
            ],
            options={
                'abstract': False,
            },
            bases=('cms.cmsplugin',),
        ),
        migrations.CreateModel(
            name='Option',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('value', models.CharField(max_length=50, verbose_name='Value')),
                ('default_value', models.BooleanField(default=False, verbose_name='Default')),
                ('field', models.ForeignKey(editable=False, to='aldryn_forms.FieldPlugin', on_delete=models.CASCADE)),
            ],
            options={
            },
            bases=(models.Model,),
        ),
        migrations.CreateModel(
            name='TextAreaFieldPlugin',
            fields=[
                ('cmsplugin_ptr', models.OneToOneField(parent_link=True, auto_created=True, primary_key=True, serialize=False, to='cms.CMSPlugin', on_delete=models.CASCADE)),
                ('label', models.CharField(max_length=50, verbose_name='Label', blank=True)),
                ('required', models.BooleanField(default=True, verbose_name='Field is required')),
                ('required_message', models.TextField(help_text='Error message displayed if the required field is left empty. Default: "This field is required".', null=True, verbose_name='Error message', blank=True)),
                ('placeholder_text', models.CharField(help_text='Default text in a form. Disappears when user starts typing. Example: "email@exmaple.com"', max_length=50, verbose_name='Placeholder text', blank=True)),
                ('help_text', models.TextField(help_text='Explanatory text displayed next to input field. Just like this one.', null=True, verbose_name='Help text', blank=True)),
                ('min_value', models.PositiveIntegerField(null=True, verbose_name='Min value', blank=True)),
                ('max_value', models.PositiveIntegerField(null=True, verbose_name='Max value', blank=True)),
                ('custom_classes', models.CharField(max_length=200, verbose_name='custom css classes', blank=True)),
                ('text_area_columns', models.PositiveIntegerField(null=True, verbose_name='columns', blank=True)),
                ('text_area_rows', models.PositiveIntegerField(null=True, verbose_name='rows', blank=True)),
            ],
            options={
                'abstract': False,
            },
            bases=('cms.cmsplugin',),
        ),
    ]



# Generated by Django 4.2.20 on 2025-04-29 12:27

from django.db import migrations

import aldryn_forms.fields


def populate_field_redirect_to(apps, schema_editor):
    if schema_editor.connection.alias != "default":
        return
    FormPlugin = apps.get_model("aldryn_forms", "FormPlugin")
    for instance in FormPlugin.objects.all():
        if instance.redirect_type == "redirect_to_page":
            if instance.redirect_page is not None:
                instance.redirect_to = {"internal_link": f"cms.page:{instance.redirect_page.pk}"}
                instance.save()
        elif instance.redirect_type == "redirect_to_url":
            instance.redirect_to = {"external_link": instance.url}
            instance.save()


class Migration(migrations.Migration):

    dependencies = [
        ('aldryn_forms', '0021_formsubmission_honeypot_filled'),
    ]

    operations = [
        migrations.AddField(
            model_name='formplugin',
            name='redirect_to',
            field=aldryn_forms.fields.AldrynFormsLinkField(blank=True, default=dict, help_text='-', null=True, verbose_name='Redirect to'),
        ),
        migrations.RunPython(populate_field_redirect_to),
    ]



from __future__ import unicode_literals

import django.core.validators
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('aldryn_forms', '0002_auto_20151014_1631'),
    ]

    operations = [
        migrations.AlterField(
            model_name='emailfieldplugin',
            name='max_value',
            field=models.PositiveIntegerField(blank=True, null=True, verbose_name='Max value', validators=[django.core.validators.MaxValueValidator(255)]),
        ),
        migrations.AlterField(
            model_name='fieldplugin',
            name='max_value',
            field=models.PositiveIntegerField(blank=True, null=True, verbose_name='Max value', validators=[django.core.validators.MaxValueValidator(255)]),
        ),
        migrations.AlterField(
            model_name='fileuploadfieldplugin',
            name='max_value',
            field=models.PositiveIntegerField(blank=True, null=True, verbose_name='Max value', validators=[django.core.validators.MaxValueValidator(255)]),
        ),
        migrations.AlterField(
            model_name='formdata',
            name='language',
            field=models.CharField(default='en', max_length=10, verbose_name='language', choices=[('de', 'German'), ('en', 'English')]),
        ),
        migrations.AlterField(
            model_name='imageuploadfieldplugin',
            name='max_value',
            field=models.PositiveIntegerField(blank=True, null=True, verbose_name='Max value', validators=[django.core.validators.MaxValueValidator(255)]),
        ),
        migrations.AlterField(
            model_name='textareafieldplugin',
            name='max_value',
            field=models.PositiveIntegerField(blank=True, null=True, verbose_name='Max value', validators=[django.core.validators.MaxValueValidator(255)]),
        ),
    ]



from typing import Optional

from django.conf import settings
from django.forms import MultiWidget, Select, URLInput
from django.utils.translation import gettext_lazy as _

from djangocms_link.fields import LinkAutoCompleteWidget, SiteAutocompleteSelect, TextInput


# Configure the LinkWidget
link_types = {
    "internal_link": _("Internal link"),
    "external_link": _("External link/anchor"),
}

allowed_link_types = tuple(link_types.keys())

# Show anchor sub-widget only for internal_link
_mapping = {key: key for key in link_types.keys()}
_mapping["anchor"] = "internal_link"


# Create the available widgets
_available_widgets = {
    "always": Select(
        choices=list(link_types.items()),
        attrs={
            "class": "js-link-widget-selector",
            "data-help": _("No destination selected. Use the dropdown to select a destination."),
        },
    ),  # Link type selector
    "external_link": URLInput(
        attrs={
            "widget": "external_link",
            "placeholder": _("https://example.com or #anchor"),
            "data-help": _(
                "Provide a link to an external URL, including the schema such as 'https://'. "
                "Optionally, add an #anchor (including the #) to scroll to."
            )
        },
    ),  # External link input
    "internal_link": LinkAutoCompleteWidget(
        attrs={
            "widget": "internal_link",
            "data-help": _("Select from available internal destinations. Optionally, add an anchor to scroll to."),
            "data-placeholder": _("Select internal destination"),
        },
    ),  # Internal link selector
    "anchor": TextInput(
        attrs={
            "widget": "anchor",
            "placeholder": _("#anchor"),
            "data-help": _("Provide an anchor to scroll to."),
        }
    ),
}


class LinkWidget(MultiWidget):
    template_name = "djangocms_link/admin/link_widget.html"
    data_pos = {}
    number_sites = None
    default_site_selector = getattr(settings, "DJANGOCMS_LINK_SITE_SELECTOR", False)

    class Media:
        js = ("djangocms_link/link-widget.js",)
        css = {"all": ("djangocms_link/link-widget.css",)}

    def __init__(self, site_selector: Optional[bool] = None):
        if site_selector is None:
            site_selector = LinkWidget.default_site_selector

        widgets = [
            widget
            for key, widget in _available_widgets.items()
            if key == "always" or _mapping[key] in link_types
        ]
        if site_selector and "internal_link" in allowed_link_types:
            index = next(
                i
                for i, widget in enumerate(widgets)
                if widget.attrs.get("widget") == "internal_link"
            )
            widgets.insert(
                index,
                SiteAutocompleteSelect(
                    attrs={
                        "class": "js-link-site-widget",
                        "widget": "site",
                        "data-placeholder": _("Select site"),
                    },
                ),
            )  # Site selector

        # Remember which widget expets its content at which position
        self.data_pos = {
            widget.attrs.get("widget"): i for i, widget in enumerate(widgets)
        }
        super().__init__(widgets)

    def get_context(self, name: str, value: Optional[str], attrs: dict) -> dict:
        if not self.is_required:
            self.widgets[0].choices = [("empty", "---------")] + self.widgets[0].choices
        context = super().get_context(name, value, attrs)
        context["widget"]["subwidgets"] = {
            widget["attrs"].get("widget", "link-type-selector"): widget
            for widget in context["widget"]["subwidgets"]
        }
        return context



# Generated by Django 1.10.8 on 2017-12-18 12:49
from __future__ import unicode_literals

from django.db import migrations, models

import aldryn_forms.utils


action_backends_choices = aldryn_forms.utils.action_backend_choices()
action_backends_choices = [(choice[0], str(choice[1])) for choice in action_backends_choices]


class Migration(migrations.Migration):

    dependencies = [
        ('aldryn_forms', '0008_auto_20170316_0845'),
    ]

    operations = [
        migrations.AddField(
            model_name='formplugin',
            name='storage_backend',
            field=models.CharField(choices=action_backends_choices, default='default', max_length=15, verbose_name='Action backend'),
        ),
    ]



from typing import TYPE_CHECKING

from django.http import HttpRequest

from rest_framework.permissions import BasePermission


if TYPE_CHECKING:  # pragma: no cover
    from .views import FormViewSet, SubmissionsViewSet


class SubmissionsPermission(BasePermission):

    def has_permission(self, request: HttpRequest, view: "SubmissionsViewSet") -> bool:
        return request.user.has_perm("aldryn_forms.view_formsubmission")


class FormPermission(BasePermission):

    def has_permission(self, request: HttpRequest, view: "FormViewSet") -> bool:
        return request.user.has_perm("aldryn_forms.view_formplugin")



import logging
import re
import smtplib
from typing import Any, Dict, List, Optional, Tuple
from urllib.parse import parse_qs, urlencode, urlparse

from django import forms
from django.apps import apps
from django.conf import settings
from django.contrib import messages
from django.contrib.admin import TabularInline
from django.core.validators import MinLengthValidator
from django.db.models import query
from django.http import HttpRequest
from django.template.loader import select_template
from django.utils.safestring import mark_safe
from django.utils.translation import gettext
from django.utils.translation import gettext_lazy as _

from cms.models.pluginmodel import CMSPlugin
from cms.plugin_base import CMSPluginBase
from cms.plugin_pool import plugin_pool
from cms.plugin_rendering import PluginContext

import markdown
from emailit.api import send_mail
from filer.models import filemodels, imagemodels
from PIL import Image

from . import models
from .constants import ALDRYN_FORMS_MULTIPLE_SUBMISSION_DURATION, ALDRYN_FORMS_POST_IDENT_NAME, MAX_IDENT_SIZE
from .forms import (
    BooleanFieldForm, CaptchaFieldForm, DateFieldForm, DateTimeFieldForm, EmailFieldForm, FileFieldForm, FormPluginForm,
    FormSubmissionBaseForm, HiddenFieldForm, ImageFieldForm, MultipleSelectFieldForm, RadioFieldForm,
    RestrictedFileField, RestrictedImageField, RestrictedMultipleFilesField, SelectFieldForm, TextAreaFieldForm,
    TextFieldForm, TimeFieldForm, URLFieldForm,
)
from .helpers import get_user_name
from .models import FieldPluginBase, FormField, SerializedFormField, SubmittedToBeSent
from .signals import form_post_save, form_pre_save
from .sizefield.utils import filesizeformat
from .utils import get_action_backends, send_email
from .validators import MaxChoicesValidator, MinChoicesValidator, is_valid_recipient


logger = logging.getLogger(__name__)


class FormElement(CMSPluginBase):
    # Don't cache anything.
    cache = False
    module = _('Forms')


class FieldContainer(FormElement):
    allow_children = True


class FormPlugin(FieldContainer):
    render_template = True
    name = _('Form')
    module = _('Form types')
    model = models.FormPlugin
    form = FormPluginForm
    filter_horizontal = ['recipients']
    ident_field_name = None
    honeypot_filled = False

    fieldsets = (
        (None, {
            'fields': (
                'name',
                'redirect_to',
                'use_form_action',
            )
        }),
        (_('Advanced Settings'), {
            'classes': ('collapse',),
            'fields': (
                'form_template',
                'error_message',
                'success_message',
                'recipients',
                'action_backend',
                'webhooks',
                'custom_classes',
                'form_attributes',
            )
        }),
    )

    def render(self, context, instance, placeholder):
        context = super().render(context, instance, placeholder)
        request = context['request']

        form = self.process_form(instance, request)

        if request.POST.get('form_plugin_id') == str(instance.id) and form.is_valid():
            context['post_success'] = True
            context['form_success_url'] = self.get_success_url(instance, form.instance.post_ident)
        context['form'] = form
        return context

    def get_render_template(self, context, instance, placeholder):
        return instance.form_template

    def form_valid(self, instance: models.FormPlugin, request: HttpRequest, form: FormSubmissionBaseForm) -> Any:
        action_backend = get_action_backends()[form.form_plugin.action_backend]()
        return action_backend.form_valid(self, instance, request, form)

    def form_invalid(self, instance: models.FormPlugin, request: HttpRequest, form: FormSubmissionBaseForm) -> None:
        if instance.error_message:
            form._add_error(message=instance.error_message)
        else:
            form._add_error(message=_(
                "The form could not be submitted. Please check that all form fields are filled in correctly."))

    def process_form(self, instance: models.FormPlugin, request: HttpRequest) -> FormSubmissionBaseForm:
        PROCESSED_FORM = "aldryn_forms_processed_forms"
        processed_forms_dict = getattr(request, PROCESSED_FORM, {})
        processed_form = processed_forms_dict.get(instance.pk)
        if processed_form is not None:
            return processed_form  # Form was already processed by middleware HandleHttpPost.

        form_class = self.get_form_class(instance)
        form_kwargs = self.get_form_kwargs(instance, request)
        form = form_class(**form_kwargs)  # django.forms.widgets.AldrynDynamicForm
        form.ident_field_name = self.ident_field_name

        processed_forms_dict[instance.pk] = form
        setattr(request, PROCESSED_FORM, processed_forms_dict)

        # Put files into fields.
        if form.files:
            for input_name in form.files.keys():
                if input_name in form.fields:
                    form.fields[input_name].files = form.files.getlist(input_name)

        if request.POST.get('form_plugin_id') == str(instance.id) and form.is_valid():
            if self.ident_field_name:
                form.cleaned_data[self.ident_field_name] = request.POST.get(self.ident_field_name, "")[:MAX_IDENT_SIZE]
            fields = [field for field in form.base_fields.values()
                      if hasattr(field, '_plugin_instance')]

            form.instance.honeypot_filled = self.honeypot_filled

            # pre save field hooks
            for field in fields:
                field._plugin_instance.form_pre_save(
                    instance=field._model_instance,
                    form=form,
                    request=request,
                )

            form_pre_save.send(
                sender=models.FormPlugin,
                instance=instance,
                form=form,
                request=request,
            )

            self.form_valid(instance, request, form)

            # post save field hooks
            for field in fields:
                field._plugin_instance.form_post_save(
                    instance=field._model_instance,
                    form=form,
                    request=request,
                )

            form_post_save.send(
                sender=models.FormPlugin,
                instance=instance,
                form=form,
                request=request,
            )
        elif request.POST.get('form_plugin_id') == str(instance.id) and request.method == 'POST':
            # only call form_invalid if request is POST and form is not valid
            self.form_invalid(instance, request, form)

        return form

    def get_form_class(self, instance):
        """
        Constructs form class basing on children plugin instances.
        """
        fields = self.get_form_fields(instance)
        formClass = (
            type(FormSubmissionBaseForm)
            ('AldrynDynamicForm', (FormSubmissionBaseForm,), fields)
        )
        return formClass

    def get_form_fields(self, instance: models.FormPlugin) -> Dict:
        form_fields = {}
        fields = instance.get_form_fields()

        for field in fields:
            plugin_instance = field.plugin_instance                     # aldryn_forms.models.FieldPlugin
            field_plugin = plugin_instance.get_plugin_class_instance()  # aldryn_forms.cms_plugins.TextField
            form_fields[field.name] = field_plugin.get_form_field(plugin_instance)
            form_fields[field.name]._cms_form_plugin = self
        return form_fields

    def get_form_kwargs(self, instance, request):
        kwargs = {
            'form_plugin': instance,
            'request': request,
        }

        if request.POST.get('form_plugin_id') == str(instance.id) and request.method in ('POST', 'PUT'):
            kwargs['data'] = request.POST.copy()
            kwargs['data']['language'] = instance.language
            kwargs['data']['form_plugin_id'] = instance.pk
            kwargs['files'] = request.FILES
        return kwargs

    def get_success_url(self, instance: models.FormPlugin, post_ident: Optional[str]) -> str:
        duration = getattr(settings, ALDRYN_FORMS_MULTIPLE_SUBMISSION_DURATION, 0)
        if duration and post_ident is not None and instance.success_url is not None:
            result = urlparse(instance.success_url)
            params = parse_qs(result.query)
            params[ALDRYN_FORMS_POST_IDENT_NAME] = post_ident
            result = result._replace(query=urlencode(params))
            return result.geturl()
        return instance.success_url

    def send_success_message(self, instance, request):
        """
        Sends a success message to the request user
        using django's contrib.messages app.
        """
        if instance.success_message:
            message = markdown.markdown(instance.success_message)
            if request.META.get('HTTP_X_REQUESTED_WITH') == "XMLHttpRequest":
                request.aldryn_forms_success_message = message
            else:
                messages.success(request, mark_safe(message))

    def save_new_submission(self, form: FormSubmissionBaseForm, post_ident: str) -> SubmittedToBeSent:
        """Save a new submission to be sent."""
        return SubmittedToBeSent.objects.create(
            name=form.instance.name,
            data=form.instance.data,
            recipients=form.instance.recipients,
            language=form.instance.language,
            form_url=form.instance.form_url,
            sent_at=form.instance.sent_at,
            post_ident=post_ident,
            honeypot_filled=form.instance.honeypot_filled
        )

    def postpone_send_notifications(
        self, instance: models.FormPlugin, form: FormSubmissionBaseForm
    ) -> List[Tuple[str, str]]:
        users = instance.recipients.exclude(email='')
        recipients = [user for user in users.iterator() if is_valid_recipient(user.email)]
        users_notified = [(get_user_name(user), user.email) for user in recipients]
        form.instance.set_recipients(users_notified)
        form.instance.set_form_data(form)

        post_ident = form.cleaned_data.get(ALDRYN_FORMS_POST_IDENT_NAME)
        if post_ident is None or post_ident.strip() == "":
            post_ident = form.generate_post_ident()
            form.initial_post_ident = post_ident
            form.cleaned_data[ALDRYN_FORMS_POST_IDENT_NAME] = post_ident
            new_instance = self.save_new_submission(form, post_ident)
            new_instance.webhooks.set(instance.webhooks.all())
        else:
            try:
                previous_submit = SubmittedToBeSent.objects.get(post_ident=post_ident)
                form.append_into_previous_submission(previous_submit)
            except SubmittedToBeSent.DoesNotExist:
                new_instance = self.save_new_submission(form, post_ident)
                new_instance.webhooks.set(instance.webhooks.all())

        return users_notified

    def send_notifications(self, instance, form):
        users = instance.recipients.exclude(email='')
        recipients = [user for user in users.iterator() if is_valid_recipient(user.email)]
        if recipients and send_email(recipients, instance, form.get_serialized_form_fields(),
                                     form.cleaned_data.items()):
            users_notified = [(get_user_name(user), user.email) for user in recipients]
        else:
            users_notified = []
        return users_notified


class FormWithIdentPlugin(FormPlugin):
    render_template = True
    name = _("Form with Ident field")
    ident_field_name = ALDRYN_FORMS_POST_IDENT_NAME


class Fieldset(FieldContainer):
    render_template = True
    name = _('Fieldset')
    model = models.FieldsetPlugin

    fieldsets = (
        (None, {
            'fields': (
                'legend',
            )
        }),
        (_('Advanced Settings'), {
            'classes': ('collapse',),
            'fields': (
                'custom_classes',
            )
        }),
    )

    def get_render_template(self, context, instance, placeholder):
        try:
            form_plugin = context['form'].form_plugin
        except KeyError:
            # technically a fieldset is not allowed outside a form.
            # unfortunately, there's no builtin way to enforce this on the cms
            form_plugin = None
        templates = self.get_template_names(instance, form_plugin)
        return select_template(templates)

    def get_template_names(self, instance, form_plugin=None):
        template_names = ['aldryn_forms/fieldset.html']

        if form_plugin:
            template = f'aldryn_forms/{form_plugin.plugin_type.lower()}/fieldset.html'
            template_names.insert(0, template)
        return template_names


class Field(FormElement):
    module = _('Form fields')
    # template name is calculated based on field
    render_template = True
    model = models.FieldPlugin

    # Custom field related attributes
    form_field = None
    form_field_widget = None
    form_field_enabled_options = [
        'label',
        'name',
        'help_text',
        'required',
        'attributes',
    ]
    form_field_disabled_options = []

    # Used to configure default fieldset in admin form
    fieldset_general_fields = [
        'label',
        'name',
        'placeholder_text',
        'required',
    ]
    fieldset_advanced_fields = [
        'attributes',
        'help_text',
        ('min_value', 'max_value',),
        'required_message',
        'custom_classes',
    ]

    def serialize_value(self, instance, value, is_confirmation=False):
        if isinstance(value, query.QuerySet):
            value = ', '.join(map(str, value))
        elif value is None:
            value = '-'
        return str(value)

    def serialize_field(self, form:  models.FormPlugin, field: FormField, is_confirmation=False):
        """Returns a (key, label, value) named tuple for the given field."""
        value = self.serialize_value(
            instance=field.plugin_instance,
            value=form.cleaned_data[field.name],
            is_confirmation=is_confirmation,
        )
        serialized_field = SerializedFormField(
            name=field.name,
            label=field.label,
            field_occurrence=field.field_occurrence,
            value=value,
            plugin_type=field.plugin_instance.plugin_type
        )
        return serialized_field

    def get_form_field(self, instance: FieldPluginBase) -> forms.Field:
        form_field_class = self.get_form_field_class(instance)
        form_field_kwargs = self.get_form_field_kwargs(instance)
        field = form_field_class(**form_field_kwargs)
        field.required = instance.required
        # allow fields access to their model plugin class instance
        field._model_instance = instance
        # and also to the plugin class instance
        field._plugin_instance = self
        field._cms_form_plugin = None
        return field

    def get_form_field_class(self, instance):
        return self.form_field

    def get_form_field_kwargs(self, instance):
        allowed_options = self.get_field_enabled_options()

        kwargs = {'widget': self.get_form_field_widget(instance)}

        if 'error_messages' in allowed_options:
            kwargs['error_messages'] = self.get_error_messages(instance=instance)
        if 'label' in allowed_options:
            kwargs['label'] = instance.label
        if 'help_text' in allowed_options:
            kwargs['help_text'] = instance.help_text
        if 'max_length' in allowed_options:
            kwargs['max_length'] = instance.max_value
        if 'required' in allowed_options:
            kwargs['required'] = instance.required
        if 'validators' in allowed_options:
            kwargs['validators'] = self.get_form_field_validators(instance)
        if 'initial_value' in allowed_options:
            kwargs['initial'] = instance.initial_value
        return kwargs

    def get_form_field_widget(self, instance):
        form_field_widget_class = self.get_form_field_widget_class(instance)
        form_field_widget_kwargs = self.get_form_field_widget_kwargs(instance)
        form_field_widget_kwargs['attrs'] = self.get_form_field_widget_attrs(instance)
        return form_field_widget_class(**form_field_widget_kwargs)

    def get_form_field_widget_class(self, instance):
        return self.form_field_widget

    def get_form_field_widget_attrs(self, instance):
        attrs = {}
        if instance.placeholder_text:
            attrs['placeholder'] = instance.placeholder_text
        if instance.custom_classes:
            attrs['class'] = instance.custom_classes
        if instance.attributes:
            attrs.update(instance.attributes)
        return attrs

    def get_form_field_widget_kwargs(self, instance):
        return {}

    def render(self, context: PluginContext, instance: models.FieldPlugin, placeholder: str) -> PluginContext:
        context = super().render(context, instance, placeholder)

        form = context.get('form')

        if form and hasattr(form, 'form_plugin'):
            form_plugin = form.form_plugin
            field_name = form_plugin.get_form_field_name(field=instance)
            if not form[field_name].is_hidden:
                # Do not render hidden fields because they are rendered in template aldryn_forms/form.html
                # in section {% for field in form.hidden_fields %}
                context['field'] = form[field_name]
        return context

    def get_render_template(self, context, instance, placeholder):
        try:
            form_plugin = context['form'].form_plugin
        except KeyError:
            # technically a field is not allowed outside a form.
            # unfortunately, there's no builtin way to enforce this on the cms
            form_plugin = None
        templates = self.get_template_names(instance, form_plugin)
        return select_template(templates)

    def get_fieldsets(self, request, obj=None):
        if self.fieldsets or self.fields:
            # Allows overriding using fieldsets or fields. If you do that none
            # of the automatic stuff kicks in and you have to take care of
            # declaring all fields you want on the form!
            # This ends up having the same behaviour as declared_fieldsets in
            # Django <1.9 had.
            return super().get_fieldsets(request, obj=obj)

        fieldsets = [
            (None, {'fields': list(self.fieldset_general_fields)}),
        ]

        if self.fieldset_advanced_fields:
            fieldsets.append(
                (
                    _('Advanced Settings'), {
                        'classes': ('collapse',),
                        'fields': list(self.fieldset_advanced_fields),
                    }
                ))
        return fieldsets

    def get_error_messages(self, instance):
        if instance.required_message:
            return {'required': instance.required_message}
        else:
            return {}

    def get_form_field_validators(self, instance):
        return []

    def get_field_enabled_options(self):
        enabled_options = self.form_field_enabled_options
        disabled_options = self.form_field_disabled_options
        return [option for option in enabled_options if option not in disabled_options]

    def get_template_names(self, instance, form_plugin=None):
        template_names = [
            f'aldryn_forms/fields/{instance.field_type}.html',
            'aldryn_forms/field.html',
        ]

        if form_plugin:
            template = 'aldryn_forms/{}/fields/{}.html'.format(
                form_plugin.plugin_type.lower(),
                instance.field_type,
            )
            template_names.insert(0, template)
        return template_names

    # hooks to allow processing of form data per field
    def form_pre_save(self, instance, form, **kwargs):
        pass

    def form_post_save(self, instance, form, **kwargs):
        pass


class BaseTextField(Field):
    form = TextFieldForm
    form_field = forms.CharField
    form_field_widget = forms.CharField.widget
    form_field_widget_input_type = 'text'
    form_field_enabled_options = [
        'label',
        'name',
        'help_text',
        'required',
        'max_length',
        'error_messages',
        'validators',
        'placeholder',
        'initial_value',
    ]

    def get_form_field_validators(self, instance):
        validators = []

        if instance.min_value:
            validators.append(MinLengthValidator(instance.min_value))
        return validators

    def get_form_field_widget_attrs(self, instance):
        attrs = super().get_form_field_widget_attrs(instance)
        attrs['type'] = self.form_field_widget_input_type
        return attrs


class TextField(BaseTextField):
    name = _('Text Field')


class HoneypotCharField(forms.CharField):

    def clean(self, value):
        cleaned_value = super().clean(value)
        if cleaned_value:
            logger.info(f'Post disabled due to Honeypot "{self.label}" value: "{value}"')
            self._cms_form_plugin.honeypot_filled = True
        return cleaned_value


class HoneypotField(BaseTextField):
    name = _('Honeypot Field')
    form = forms.ModelForm
    form_field = HoneypotCharField
    form_field_widget = HoneypotCharField.widget
    form_field_enabled_options = [
        'label',
        'name',
        'help_text',
        'validators',
        'placeholder',
        'initial_value',
    ]
    fieldset_general_fields = [
        'label',
        'name',
        'placeholder_text',
    ]
    fieldset_advanced_fields = [
        'attributes',
        'help_text',
        'custom_classes',
    ]


class TextAreaField(BaseTextField):
    name = _('Text Area Field')
    model = models.TextAreaFieldPlugin
    form = TextAreaFieldForm
    form_field_widget = forms.Textarea
    fieldset_general_fields = [
        'label',
        'name',
        'placeholder_text',
        ('text_area_rows', 'text_area_columns',),
        'required',
    ]
    fieldset_advanced_fields = [
        'help_text',
        ('min_value', 'max_value',),
        'required_message',
        'custom_classes',
    ]

    def get_form_field_widget(self, instance):
        widget = super().get_form_field_widget(instance)

        # django adds the cols and rows attributes by default.
        # remove them if the user did not specify a value for them.
        if not instance.text_area_columns:
            del widget.attrs['cols']

        if not instance.text_area_rows:
            del widget.attrs['rows']
        return widget

    def get_form_field_widget_attrs(self, instance):
        attrs = super().get_form_field_widget_attrs(instance)

        if instance.text_area_columns:
            attrs['cols'] = instance.text_area_columns
        if instance.text_area_rows:
            attrs['rows'] = instance.text_area_rows
        return attrs


class HiddenField(BaseTextField):
    name = _('Hidden Field')
    form = HiddenFieldForm
    form_field_widget_input_type = 'hidden'
    fieldset_general_fields = ['name', 'initial_value']
    fieldset_advanced_fields = []


class PhoneField(BaseTextField):
    name = _('Phone Field')
    form_field_widget_input_type = 'phone'


class DateField(BaseTextField):
    name = _('Date Field')
    model = models.DateFieldPlugin
    form = DateFieldForm
    form_field_widget_input_type = 'date'
    fieldset_general_fields = [
        'label',
        'name',
        'required',
    ]
    fieldset_advanced_fields = [
        ('earliest_date', 'latest_date', 'input_step'),
        'attributes',
        'help_text',
        'required_message',
        'custom_classes',
    ]

    def get_form_field_widget_attrs(self, instance):
        attrs = super().get_form_field_widget_attrs(instance)
        if instance.earliest_date is not None:
            attrs['min'] = instance.earliest_date.isoformat()
        if instance.latest_date is not None:
            attrs['max'] = instance.latest_date.isoformat()
        if instance.input_step is not None:
            attrs['step'] = instance.input_step
        return attrs


class DateTimeLocalField(BaseTextField):
    name = _('Datetime local Field')
    model = models.DateTimeLocalFieldPlugin
    form = DateTimeFieldForm
    form_field_widget_input_type = 'datetime-local'
    fieldset_general_fields = [
        'label',
        'name',
        'required',
    ]
    fieldset_advanced_fields = [
        ('earliest_datetime', 'latest_datetime', 'input_step'),
        'attributes',
        'help_text',
        'required_message',
        'custom_classes',
    ]

    def get_form_field_widget_attrs(self, instance):
        attrs = super().get_form_field_widget_attrs(instance)
        if instance.earliest_datetime is not None:
            attrs['min'] = instance.earliest_datetime.replace(tzinfo=None).isoformat(timespec='minutes')
        if instance.latest_datetime is not None:
            attrs['max'] = instance.latest_datetime.replace(tzinfo=None).isoformat(timespec='minutes')
        if instance.input_step is not None:
            attrs['step'] = instance.input_step
        return attrs


class TimeField(BaseTextField):
    name = _('Time Field')
    model = models.TimeFieldPlugin
    form = TimeFieldForm
    form_field_widget_input_type = 'time'
    fieldset_advanced_fields = [
        ('earliest_time', 'latest_time', 'input_step'),
        'data_list',
        'readonly',
        'attributes',
        'help_text',
        'required_message',
        'custom_classes',
    ]

    def get_form_field_widget_attrs(self, instance):
        attrs = super().get_form_field_widget_attrs(instance)
        if instance.earliest_time is not None:
            attrs['min'] = instance.earliest_time.isoformat(timespec='minutes')
        if instance.latest_time is not None:
            attrs['max'] = instance.latest_time.isoformat(timespec='minutes')
        if instance.input_step is not None:
            attrs['step'] = instance.input_step
        if instance.data_list is not None:
            attrs['list'] = instance.data_list
        if instance.readonly:
            attrs['readonly'] = True
        return attrs


class NumberField(BaseTextField):
    name = _('Number Field')
    form_field_widget_input_type = 'number'


class EmailField(BaseTextField):
    name = _('Email Field')
    model = models.EmailFieldPlugin
    form = EmailFieldForm
    form_field = forms.EmailField
    form_field_widget = forms.EmailInput
    form_field_widget_input_type = 'email'
    fieldset_advanced_fields = [
        'email_send_notification',
        'email_subject',
        'email_body',
    ] + Field.fieldset_advanced_fields
    email_template_base = 'aldryn_forms/emails/user/notification'

    def send_notification_email(self, email, form, form_field_instance):
        context = {
            'form_name': form.instance.name,
            'form_data': form.get_serialized_form_fields(is_confirmation=True),
            'body_text': form_field_instance.email_body,
        }
        try:
            send_mail(
                recipients=[email],
                context=context,
                subject=form_field_instance.email_subject,
                template_base=self.email_template_base
            )
        except smtplib.SMTPException as err:
            logger.error(err)

    def form_post_save(self, instance: models.EmailFieldPlugin, form: forms.Form, **kwargs):
        # form: <class 'django.forms.widgets.AldrynDynamicForm'>
        # form.instance: <class 'aldryn_forms.models.FormSubmission'>
        if form.instance.honeypot_filled:
            return
        field_name = form.form_plugin.get_form_field_name(field=instance)

        email = form.cleaned_data.get(field_name)

        if email and instance.email_send_notification:
            self.send_notification_email(email, form, instance)


class FileField(Field):
    name = _('File upload field')
    model = models.FileUploadFieldPlugin

    form = FileFieldForm
    form_field = RestrictedFileField
    form_field_widget = RestrictedFileField.widget
    form_field_enabled_options = [
        'label',
        'help_text',
        'required',
        'error_messages',
        'validators',
    ]
    fieldset_general_fields = [
        'upload_to',
        'is_private',
    ] + Field.fieldset_general_fields
    fieldset_advanced_fields = [
        'help_text',
        'max_size',
        'accepted_types',
        'required_message',
        'custom_classes',
        'enable_js',
    ]

    def get_form_field_widget_attrs(self, instance):
        attrs = super().get_form_field_widget_attrs(instance)
        if hasattr(instance, 'accepted_types') and instance.accepted_types:
            attrs['accept'] = ",".join(re.split(r"\s+", instance.accepted_types))
        if instance.max_size:
            attrs['data-max_size'] = instance.max_size
        if instance.enable_js:
            attrs['data-enable_js'] = "on"
        return attrs

    def get_form_field_kwargs(self, instance):
        kwargs = super().get_form_field_kwargs(instance)
        if instance.max_size:
            if 'help_text' in kwargs:
                kwargs['help_text'] = kwargs['help_text'].replace(
                    'MAXSIZE', filesizeformat(instance.max_size))
            kwargs['max_size'] = instance.max_size
        if hasattr(instance, 'accepted_types') and instance.accepted_types:
            kwargs['accepted_types'] = re.split(r"\s+", instance.accepted_types)
        return kwargs

    def serialize_value(self, instance, value, is_confirmation=False):
        links = []
        for item in value:
            if item:
                links.append(item.original_filename if is_confirmation else item.absolute_uri)
        return "\n".join(links)

    def form_pre_save(self, instance, form, **kwargs):
        """Save the uploaded file to django-filer

        The type of model (file or image) is automatically chosen by trying to
        open the uploaded file.
        """
        request = kwargs['request']

        filer_file_instances = []
        field_name = form.form_plugin.get_form_field_name(field=instance)

        for uploaded_file in request.FILES.getlist(field_name):
            if uploaded_file.content_type == 'image/svg+xml':
                model = imagemodels.Image
            else:
                try:
                    with Image.open(uploaded_file) as img:
                        img.verify()
                except:  # noqa
                    model = filemodels.File
                else:
                    model = imagemodels.Image

            filer_file = model(
                folder=instance.upload_to,
                file=uploaded_file,
                name=uploaded_file.name,
                original_filename=uploaded_file.name,
                is_public=not instance.is_private,
            )
            filer_file.save()

            # NOTE: This is a hack to make the full URL available later when we
            # need to serialize this field. We avoid to serialize it here directly
            # as we could still need access to the original filer File instance.
            filer_file.absolute_uri = request.build_absolute_uri(filer_file.url)
            filer_file_instances.append(filer_file)

        form.cleaned_data[field_name] = filer_file_instances


class MultipleFilesField(FileField):

    name = _('Multiple files upload field')
    model = models.MultipleFilesUploadFieldPlugin
    form_field = RestrictedMultipleFilesField
    form_field_widget = RestrictedMultipleFilesField.widget
    fieldset_advanced_fields = [
        'help_text',
        'max_size',
        'max_files',
        'accepted_types',
        'required_message',
        'custom_classes',
        'enable_js',
    ]

    def get_form_field_widget_attrs(self, instance):
        attrs = super().get_form_field_widget_attrs(instance)
        attrs['multiple'] = True
        if instance.max_files:
            attrs['data-max_files'] = instance.max_files
        return attrs

    def get_form_field_kwargs(self, instance):
        kwargs = super().get_form_field_kwargs(instance)
        if instance.max_files:
            kwargs['max_files'] = instance.max_files
        return kwargs


class ImageField(FileField):
    name = _('Image upload field')
    model = models.ImageUploadFieldPlugin

    form = ImageFieldForm
    form_field = RestrictedImageField
    form_field_widget = RestrictedImageField.widget
    fieldset_general_fields = [
        'upload_to',
        'is_private',
    ] + Field.fieldset_general_fields
    fieldset_advanced_fields = [
        'help_text',
        'max_size',
        ('max_width', 'max_height',),
        'required_message',
        'custom_classes',
    ]

    def get_form_field_kwargs(self, instance):
        kwargs = super().get_form_field_kwargs(instance)

        if instance.max_width:
            if 'help_text' in kwargs:
                kwargs['help_text'] = kwargs['help_text'].replace(
                    'MAXWIDTH', str(instance.max_width))
            kwargs['max_width'] = instance.max_width

        if instance.max_height:
            if 'help_text' in kwargs:
                kwargs['help_text'] = kwargs['help_text'].replace(
                    'MAXHEIGHT', str(instance.max_height))
            kwargs['max_height'] = instance.max_height

        return kwargs


class BooleanField(Field):
    # checkbox field
    # I add the above because searching for "checkbox" should give me this plugin :)
    name = _('Yes/No Field')

    form = BooleanFieldForm
    form_field = forms.BooleanField
    form_field_widget = form_field.widget
    form_field_enabled_options = [
        'label',
        'name',
        'attributes',
        'help_text',
        'required',
        'error_messages',
    ]
    fieldset_general_fields = [
        'label', 'name', 'required',
    ]
    fieldset_advanced_fields = [
        'attributes',
        'help_text',
        'required_message',
        'custom_classes',
    ]

    def serialize_value(self, instance, value, is_confirmation=False):
        return gettext('Yes') if value else gettext('No')


class SelectOptionInline(TabularInline):
    model = models.Option


class SelectField(Field):
    name = _('Select Field')

    form = SelectFieldForm
    form_field = forms.ModelChoiceField
    form_field_widget = form_field.widget
    form_field_enabled_options = [
        'label',
        'name',
        'attributes',
        'help_text',
        'required',
        'error_messages',
    ]
    fieldset_general_fields = [
        'label', 'name', 'required',
    ]
    fieldset_advanced_fields = [
        'attributes',
        'help_text',
        'required_message',
        'custom_classes',
    ]

    inlines = [SelectOptionInline]

    def get_form_field_kwargs(self, instance):
        kwargs = super().get_form_field_kwargs(instance)
        kwargs['queryset'] = instance.option_set.all()
        for opt in kwargs['queryset']:
            if opt.default_value:
                kwargs['initial'] = opt.pk
                break
        return kwargs


class MultipleSelectField(SelectField):
    name = _('Multiple Select Field')

    form = MultipleSelectFieldForm
    form_field = forms.ModelMultipleChoiceField
    form_field_widget = forms.CheckboxSelectMultiple
    form_field_enabled_options = [
        'label',
        'name',
        'attributes',
        'help_text',
        'required',
        'validators',
    ]
    fieldset_general_fields = [
        'label', 'name', 'required',
    ]
    fieldset_advanced_fields = [
        'attributes',
        ('min_value', 'max_value'),
    ]

    def get_form_field_validators(self, instance):
        validators = []
        if instance.min_value:
            validators.append(MinChoicesValidator(limit_value=instance.min_value))
        if instance.max_value:
            validators.append(MaxChoicesValidator(limit_value=instance.max_value))
        return validators

    def get_form_field_kwargs(self, instance):
        kwargs = super().get_form_field_kwargs(instance)
        if hasattr(instance, 'min_value') and instance.min_value == 0:
            kwargs['required'] = False

        kwargs['initial'] = [o.pk for o in kwargs['queryset'] if o.default_value]
        return kwargs


class MultipleCheckboxSelectField(MultipleSelectField):
    name = _('Multiple Checkbox Field')


class RadioSelectField(Field):
    name = _('Radio Select Field')

    form = RadioFieldForm
    form_field = forms.ModelChoiceField
    form_field_widget = forms.RadioSelect
    form_field_enabled_options = [
        'label',
        'name',
        'attributes',
        'help_text',
        'required',
        'error_messages',
    ]
    fieldset_general_fields = [
        'label', 'name', 'required',
    ]
    fieldset_advanced_fields = [
        'attributes',
        'help_text',
        'required_message',
        'custom_classes',
    ]

    inlines = [SelectOptionInline]

    def get_form_field_kwargs(self, instance):
        kwargs = super().get_form_field_kwargs(instance)
        kwargs['queryset'] = instance.option_set.all()
        kwargs['empty_label'] = None
        for opt in kwargs['queryset']:
            if opt.default_value:
                kwargs['initial'] = opt.pk
                break
        return kwargs


class URLField(BaseTextField):
    name = _('URL Field')
    form_field_widget_input_type = 'url'
    model = models.URLFieldPlugin
    form = URLFieldForm
    fieldset_general_fields = [
        'label',
        'name',
        'placeholder_text',
        'required',
        'help_text',
    ]
    fieldset_advanced_fields = [
        'required_message',
        ('min_value', 'max_value',),
        'list',
        'pattern',
        'readonly',
        'size',
        'spellcheck',
        'custom_classes',
        'attributes',
    ]


try:
    if not apps.is_installed("captcha"):  # django-simple-captcha
        raise ImportError('Module "captcha" is not in INSTALLED_APPS.')
    from captcha.fields import CaptchaField, CaptchaTextInput
except ImportError:
    pass
else:
    # Don't like doing this. But we shouldn't force captcha.
    class CaptchaField(Field):
        name = _('Captcha Field')
        form = CaptchaFieldForm
        form_field = CaptchaField
        form_field_widget = CaptchaTextInput
        form_field_enabled_options = ['label', 'error_messages']
        fieldset_general_fields = [
            'label',
        ]
        fieldset_advanced_fields = [
            'required_message',
        ]

        def serialize_field(self, *args, **kwargs):
            # None means don't serialize me
            return None

    plugin_pool.register_plugin(CaptchaField)


class SubmitButton(FormElement):
    render_template = 'aldryn_forms/submit_button.html'
    name = _('Submit Button')
    model = models.FormButtonPlugin


@plugin_pool.register_plugin
class HideContentWhenPostPlugin(CMSPluginBase):
    module = _('Forms')
    name = _("Hide content after submitting a form")
    model = CMSPlugin
    render_template = "aldryn_forms/hide_content_when_post.html"
    cache = False
    allow_children = True

    def render(self, context, instance, placeholder):
        context = super().render(context, instance, placeholder)
        context['display_content'] = context['request'].method != "POST"
        return context


plugin_pool.register_plugin(BooleanField)
plugin_pool.register_plugin(EmailField)
plugin_pool.register_plugin(FileField)
plugin_pool.register_plugin(MultipleFilesField)
plugin_pool.register_plugin(HiddenField)
plugin_pool.register_plugin(PhoneField)
plugin_pool.register_plugin(DateField)
plugin_pool.register_plugin(TimeField)
plugin_pool.register_plugin(DateTimeLocalField)
plugin_pool.register_plugin(NumberField)
plugin_pool.register_plugin(ImageField)
plugin_pool.register_plugin(Fieldset)
plugin_pool.register_plugin(FormPlugin)
plugin_pool.register_plugin(FormWithIdentPlugin)
plugin_pool.register_plugin(MultipleSelectField)
plugin_pool.register_plugin(MultipleCheckboxSelectField)
plugin_pool.register_plugin(RadioSelectField)
plugin_pool.register_plugin(SelectField)
plugin_pool.register_plugin(SubmitButton)
plugin_pool.register_plugin(TextAreaField)
plugin_pool.register_plugin(TextField)
plugin_pool.register_plugin(HoneypotField)
plugin_pool.register_plugin(URLField)



# Generated by Django 5.2.6 on 2025-11-12 09:30

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("aldryn_forms", "0025_alter_formplugin_action_backend_and_language"),
    ]

    operations = [
        migrations.AddField(
            model_name="fileuploadfieldplugin",
            name="is_private",
            field=models.BooleanField(
                default=False, help_text="The file is private.", verbose_name="Private"
            ),
        ),
        migrations.AddField(
            model_name="imageuploadfieldplugin",
            name="is_private",
            field=models.BooleanField(
                default=False, help_text="The file is private.", verbose_name="Private"
            ),
        ),
        migrations.AddField(
            model_name="multiplefilesuploadfieldplugin",
            name="is_private",
            field=models.BooleanField(
                default=False, help_text="The files are private.", verbose_name="Private"
            ),
        ),
    ]



from rest_framework import serializers

from aldryn_forms.models import FormPlugin, FormSubmission


class FormSubmissionSerializer(serializers.HyperlinkedModelSerializer):

    hostname = serializers.SerializerMethodField('set_hostname')

    class Meta:
        model = FormSubmission
        fields = ['hostname', 'name', 'language', 'sent_at', 'form_recipients', 'form_data']

    def set_hostname(self, instance: FormSubmission) -> str:
        return self.context.get("hostname", "testserver")


class FormSerializer(serializers.HyperlinkedModelSerializer):

    class Meta:
        model = FormPlugin
        fields = ['name']



# Generated by Django 4.2.20 on 2025-03-19 12:13

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('aldryn_forms', '0019_submittedtobesent_formsubmission_post_ident_and_more'),
    ]

    operations = [
        migrations.CreateModel(
            name='Webhook',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=255, unique=True, verbose_name='Name')),
                ('url', models.URLField(unique=True, verbose_name='Webhook URL')),
                ('method', models.CharField(choices=[('post', 'POST'), ('json', 'JSON')], default=('post', 'POST'), max_length=20, verbose_name='Data transfer method')),
                ('transform', models.JSONField(blank=True, null=True, verbose_name='Data transform')),
            ],
        ),
        migrations.AddField(
            model_name='formplugin',
            name='webhooks',
            field=models.ManyToManyField(blank=True, to='aldryn_forms.webhook'),
        ),
        migrations.AddField(
            model_name='formsubmission',
            name='webhooks',
            field=models.ManyToManyField(blank=True, to='aldryn_forms.webhook'),
        ),
        migrations.AddField(
            model_name='submittedtobesent',
            name='webhooks',
            field=models.ManyToManyField(blank=True, to='aldryn_forms.webhook'),
        ),
    ]



# Generated by Django 4.2.3 on 2023-07-24 09:18

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("cms", "0022_auto_20180620_1551"),
        ("aldryn_forms", "0016_date_time_fields"),
    ]

    operations = [
        migrations.AlterField(
            model_name="datefieldplugin",
            name="cmsplugin_ptr",
            field=models.OneToOneField(
                on_delete=django.db.models.deletion.CASCADE,
                parent_link=True,
                primary_key=True,
                related_name="%(app_label)s_%(class)s",
                serialize=False,
                to="cms.cmsplugin",
            ),
        ),
        migrations.AlterField(
            model_name="datetimelocalfieldplugin",
            name="cmsplugin_ptr",
            field=models.OneToOneField(
                on_delete=django.db.models.deletion.CASCADE,
                parent_link=True,
                primary_key=True,
                related_name="%(app_label)s_%(class)s",
                serialize=False,
                to="cms.cmsplugin",
            ),
        ),
        migrations.AlterField(
            model_name="emailfieldplugin",
            name="cmsplugin_ptr",
            field=models.OneToOneField(
                on_delete=django.db.models.deletion.CASCADE,
                parent_link=True,
                primary_key=True,
                related_name="%(app_label)s_%(class)s",
                serialize=False,
                to="cms.cmsplugin",
            ),
        ),
        migrations.AlterField(
            model_name="fieldplugin",
            name="cmsplugin_ptr",
            field=models.OneToOneField(
                on_delete=django.db.models.deletion.CASCADE,
                parent_link=True,
                primary_key=True,
                related_name="%(app_label)s_%(class)s",
                serialize=False,
                to="cms.cmsplugin",
            ),
        ),
        migrations.AlterField(
            model_name="fieldsetplugin",
            name="cmsplugin_ptr",
            field=models.OneToOneField(
                on_delete=django.db.models.deletion.CASCADE,
                parent_link=True,
                primary_key=True,
                related_name="%(app_label)s_%(class)s",
                serialize=False,
                to="cms.cmsplugin",
            ),
        ),
        migrations.AlterField(
            model_name="fileuploadfieldplugin",
            name="cmsplugin_ptr",
            field=models.OneToOneField(
                on_delete=django.db.models.deletion.CASCADE,
                parent_link=True,
                primary_key=True,
                related_name="%(app_label)s_%(class)s",
                serialize=False,
                to="cms.cmsplugin",
            ),
        ),
        migrations.AlterField(
            model_name="fileuploadfieldplugin",
            name="enable_js",
            field=models.BooleanField(
                blank=True,
                help_text="Enable javascript to view files for upload.",
                null=True,
                verbose_name="Enable js",
            ),
        ),
        migrations.AlterField(
            model_name="formbuttonplugin",
            name="cmsplugin_ptr",
            field=models.OneToOneField(
                on_delete=django.db.models.deletion.CASCADE,
                parent_link=True,
                primary_key=True,
                related_name="%(app_label)s_%(class)s",
                serialize=False,
                to="cms.cmsplugin",
            ),
        ),
        migrations.AlterField(
            model_name="formplugin",
            name="action_backend",
            field=models.CharField(
                choices=[
                    ("default", "Default"),
                    ("email_only", "Email only"),
                    ("none", "None"),
                ],
                default="default",
                max_length=15,
                verbose_name="Action backend",
            ),
        ),
        migrations.AlterField(
            model_name="formplugin",
            name="cmsplugin_ptr",
            field=models.OneToOneField(
                on_delete=django.db.models.deletion.CASCADE,
                parent_link=True,
                primary_key=True,
                related_name="%(app_label)s_%(class)s",
                serialize=False,
                to="cms.cmsplugin",
            ),
        ),
        migrations.AlterField(
            model_name="formsubmission",
            name="id",
            field=models.BigAutoField(
                auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
            ),
        ),
        migrations.AlterField(
            model_name="imageuploadfieldplugin",
            name="cmsplugin_ptr",
            field=models.OneToOneField(
                on_delete=django.db.models.deletion.CASCADE,
                parent_link=True,
                primary_key=True,
                related_name="%(app_label)s_%(class)s",
                serialize=False,
                to="cms.cmsplugin",
            ),
        ),
        migrations.AlterField(
            model_name="imageuploadfieldplugin",
            name="enable_js",
            field=models.BooleanField(
                blank=True,
                help_text="Enable javascript to view files for upload.",
                null=True,
                verbose_name="Enable js",
            ),
        ),
        migrations.AlterField(
            model_name="multiplefilesuploadfieldplugin",
            name="cmsplugin_ptr",
            field=models.OneToOneField(
                on_delete=django.db.models.deletion.CASCADE,
                parent_link=True,
                primary_key=True,
                related_name="%(app_label)s_%(class)s",
                serialize=False,
                to="cms.cmsplugin",
            ),
        ),
        migrations.AlterField(
            model_name="multiplefilesuploadfieldplugin",
            name="enable_js",
            field=models.BooleanField(
                blank=True,
                help_text="Enable javascript to view files for upload.",
                null=True,
                verbose_name="Enable js",
            ),
        ),
        migrations.AlterField(
            model_name="option",
            name="id",
            field=models.BigAutoField(
                auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
            ),
        ),
        migrations.AlterField(
            model_name="textareafieldplugin",
            name="cmsplugin_ptr",
            field=models.OneToOneField(
                on_delete=django.db.models.deletion.CASCADE,
                parent_link=True,
                primary_key=True,
                related_name="%(app_label)s_%(class)s",
                serialize=False,
                to="cms.cmsplugin",
            ),
        ),
        migrations.AlterField(
            model_name="timefieldplugin",
            name="cmsplugin_ptr",
            field=models.OneToOneField(
                on_delete=django.db.models.deletion.CASCADE,
                parent_link=True,
                primary_key=True,
                related_name="%(app_label)s_%(class)s",
                serialize=False,
                to="cms.cmsplugin",
            ),
        ),
    ]



from __future__ import unicode_literals

from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('aldryn_forms', '0003_auto_20151207_1038'),
    ]

    operations = [
        migrations.CreateModel(
            name='FormSubmission',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('name', models.CharField(verbose_name='form name', max_length=50, editable=False, db_index=True)),
                ('data', models.TextField(editable=False, blank=True)),
                ('recipients', models.TextField(help_text='People who got a notification when form was submitted.', verbose_name='users notified', editable=False, blank=True)),
                ('language', models.CharField(default=settings.LANGUAGE_CODE, max_length=10, verbose_name='form language', choices=settings.LANGUAGES)),
                ('form_url', models.CharField(max_length=255, verbose_name='form url', blank=True)),
                ('sent_at', models.DateTimeField(auto_now_add=True)),
            ],
            options={
                'verbose_name': 'Form submission',
                'verbose_name_plural': 'Form submissions',
            },
        ),
        migrations.AlterModelOptions(
            name='formdata',
            options={'verbose_name': 'Form submission (Old)', 'verbose_name_plural': 'Form submissions (Old)'},
        ),
    ]



from typing import Callable

from django.contrib.sites.models import Site
from django.http.response import Http404

from django_filters import rest_framework as filters
from rest_framework import viewsets
from rest_framework.response import Response

from aldryn_forms.models import FormPlugin, FormSubmission

from .pagination import AldrynFormsPagination
from .permissions import FormPermission, SubmissionsPermission
from .serializers import FormSerializer, FormSubmissionSerializer


class SubmissionFilter(filters.FilterSet):
    sent_at_period = filters.DateRangeFilter(field_name='sent_at', label="Sent at date range")
    sent_at_range = filters.DateFromToRangeFilter(field_name='sent_at', label="Sent at date from to")
    sent_at_range_time = filters.DateTimeFromToRangeFilter(field_name='sent_at', label="Sent at datetime from to")

    class Meta:
        model = FormSubmission
        fields = ('name', 'language')


class SanitizeGetObjectMixin:

    get_object: Callable
    get_serializer: Callable

    def retrieve(self, request, *args, **kwargs):
        try:
            instance = self.get_object()
        except Http404:
            data = {"error": {"message": "Object not found."}}
            return Response(data, 400)  # Note: Code 404 cannot be used because it will return a "Not Found page".
        serializer = self.get_serializer(instance)
        return Response(serializer.data)


class SubmissionsViewSet(SanitizeGetObjectMixin, viewsets.ReadOnlyModelViewSet):
    authentication_classes = []
    permission_classes = [SubmissionsPermission]
    queryset = FormSubmission.objects.filter(post_ident__isnull=True).order_by('-sent_at')
    serializer_class = FormSubmissionSerializer
    paginator = AldrynFormsPagination()
    filter_backends = (filters.DjangoFilterBackend,)
    filterset_class = SubmissionFilter

    def get_serializer_context(self):
        context = super().get_serializer_context()
        site = Site.objects.first()
        context["hostname"] = site.domain
        return context


class FormViewSet(SanitizeGetObjectMixin, viewsets.ReadOnlyModelViewSet):
    authentication_classes = []
    permission_classes = [FormPermission]
    queryset = FormPlugin.objects.distinct("name").order_by('name')
    serializer_class = FormSerializer
    paginator = AldrynFormsPagination()



from typing import Dict

from django import template
from django.forms.boundfield import BoundField
from django.utils import encoding
from django.utils.safestring import mark_safe

import markdown as markdown_module

from aldryn_forms.models import FieldPluginBase


register = template.Library()


@register.simple_tag(takes_context=True)
def render_notification_text(context, email_notification, email_type):
    text_context = context.get('text_context')

    if not text_context or not email_notification:
        return

    render_func = 'render_%s' % email_type
    message = getattr(email_notification, render_func)(context=text_context)
    return mark_safe(message)


def _build_kwargs(field: BoundField, instance: FieldPluginBase) -> Dict[str, str]:
    kwargs: Dict[str, str] = {}
    if instance.custom_classes:
        kwargs["class"] = instance.custom_classes
    if "class" in kwargs and field.errors:
        if kwargs["class"]:
            kwargs["class"] += " "
        kwargs["class"] += "has-error"
    return kwargs


@register.simple_tag()
def render_form_widget(field: BoundField, instance: FieldPluginBase):
    markup = field.as_widget(attrs=_build_kwargs(field, instance))
    return mark_safe(markup)


@register.simple_tag()
def render_url_field(field: BoundField, instance: FieldPluginBase, **kwargs):
    kwargs = _build_kwargs(field, instance)
    if instance.list:
        kwargs["list"] = instance.get_html_id_list()
    if instance.min_value is not None:
        kwargs["minlength"] = instance.min_value
    for name in ("pattern", "readonly", "size", "spellcheck"):
        value = getattr(instance, name)
        if value:
            kwargs[name] = value
    markup = field.as_widget(attrs=kwargs)
    return mark_safe(markup)


@register.filter()
def force_text(val):
    return encoding.force_str(val)


@register.filter()
def force_text_list(val):
    return [encoding.force_str(v) for v in val]


@register.filter()
def markdown(text):
    return mark_safe(markdown_module.markdown(text))



from django.conf import settings
from django.contrib.sites.models import Site
from django.core.management.base import BaseCommand
from django.utils.timezone import now as django_timezone_now
from django.utils.timezone import timedelta

from aldryn_forms.api.webhook import trigger_webhooks
from aldryn_forms.constants import ALDRYN_FORMS_MULTIPLE_SUBMISSION_DURATION
from aldryn_forms.models import SubmittedToBeSent
from aldryn_forms.utils import send_postponed_notifications


class Command(BaseCommand):
    help = "Send postponed emails."

    def handle(self, *args, **options):
        duration = getattr(settings, ALDRYN_FORMS_MULTIPLE_SUBMISSION_DURATION, 0)
        if duration:
            expire = django_timezone_now() - timedelta(minutes=duration)
            queryset = SubmittedToBeSent.objects.filter(sent_at__lt=expire)
        else:
            queryset = SubmittedToBeSent.objects.all()

        site = Site.objects.first()
        for instance in queryset:
            if not instance.honeypot_filled:
                if send_postponed_notifications(instance):
                    trigger_webhooks(instance.webhooks, instance, site.domain)
            instance.delete()



from django import forms
from django.core.validators import EMPTY_VALUES

from .utils import filesizeformat, parse_size


class FileSizeWidget(forms.TextInput):

    def render(self, name, value, attrs=None, renderer=None):
        if value:
            try:
                value = filesizeformat(value)
            except ValueError:
                pass
        return super(FileSizeWidget, self).render(name, value, attrs, renderer)

    def value_from_datadict(self, data, files, name):
        """
        Given a dictionary of data and this widget's name, returns the value
        of this widget. Returns None if it's not provided.
        """
        value = super(FileSizeWidget, self).value_from_datadict(data, files, name)
        if value not in EMPTY_VALUES:
            try:
                return parse_size(value)
            except ValueError:
                pass
        return value



import operator
import re

from django.conf import settings
from django.utils import formats
from django.utils.translation import gettext as _


SIZEFIELD_FORMAT = getattr(settings, 'SIZEFIELD_FORMAT', '{value}{unit}')

file_size_re = re.compile(r'^(?P<value>[0-9\.,]+?)\s*(?P<unit>[KMGTPEZY]?B?)$', re.IGNORECASE)
FILESIZE_UNITS = {
    'B': 1,
    'KB': 1 << 10,
    'MB': 1 << 20,
    'GB': 1 << 30,
    'TB': 1 << 40,
    'PB': 1 << 50,
    'EB': 1 << 60,
    'ZB': 1 << 70,
    'YB': 1 << 80,
}


def filesizeformat(bytes, decimals=1):
    """
    Formats the value like a 'human-readable' file size (i.e. 13 KB, 4.1 MB,
    102 bytes, etc).
    Based on django.template.defaultfilters.filesizeformat
    """

    try:
        bytes = float(bytes)
    except (TypeError, ValueError, UnicodeDecodeError):
        raise ValueError

    def filesize_number_format(value):
        return formats.number_format(round(value, decimals), decimals)

    units_list = sorted(FILESIZE_UNITS.items(), key=operator.itemgetter(1))

    value = unit = None
    len_unints_list = len(units_list)
    for i in range(1, len_unints_list):
        if bytes < units_list[i][1]:
            prev_unit = units_list[i - 1]
            value = filesize_number_format(bytes / prev_unit[1])
            unit = prev_unit[0]
            break

    if value is None:
        value = filesize_number_format(bytes / units_list[-1][1])
        unit = units_list[-1][0]

    return SIZEFIELD_FORMAT.format(value=value, unit=unit)


def parse_size(size):
    """
    @rtype int
    """
    if isinstance(size, int):
        return size

    r = file_size_re.match(size.strip())
    if r:
        clean_value = r.group("value").replace(",", ".")
        value = float(clean_value)
        unit = r.group('unit').upper()
        if not unit.endswith('B'):
            unit += 'B'
        return int(value * FILESIZE_UNITS[unit])

    # Regex pattern was not matched
    raise ValueError(_("Size '%s' has incorrect format") % size)



from django.core.serializers.json import DjangoJSONEncoder


class PrettyJsonEncoder(DjangoJSONEncoder):

    def __init__(self, *args, **kwargs):
        kwargs["indent"] = 2
        kwargs["sort_keys"] = False
        super().__init__(*args, **kwargs)



from django.contrib import messages
from django.http import HttpResponse
from django.shortcuts import redirect
from django.utils.translation import get_language_from_request, gettext

from ..compat import SessionWizardView
from .exporter import Exporter
from .forms import FormExportStep1Form, FormExportStep2Form


mimetype_map = {
    'csv': 'text/csv',
    'xlsx': 'application/vnd.ms-excel',
    'xls': 'application/vnd.ms-excel',
    'tsv': 'text/tab-separated-values',
    'html': 'text/html',
    'yaml': 'text/yaml',
    'json': 'application/json',
}


class FormExportWizardView(SessionWizardView):
    admin = None
    file_type = None
    form_list = [
        FormExportStep1Form,
        FormExportStep2Form,
    ]

    template_name = 'admin/aldryn_forms/export_wizard.html'

    def get_context_data(self, form, **kwargs):
        context = super(FormExportWizardView, self).get_context_data(form, **kwargs)
        context.update(self.admin.get_admin_context(form=form, title='Export'))
        return context

    def get_form_initial(self, step):
        initial = super(FormExportWizardView, self).get_form_initial(step)

        if step == self.steps.first:
            initial['language'] = get_language_from_request(
                request=self.request,
                check_path=True
            )
        return initial

    def get_form_kwargs(self, step=None):
        """
        Returns the keyword arguments for instantiating the form
        (or formset) on the given step.
        """
        kwargs = super(FormExportWizardView, self).get_form_kwargs(step)

        if step == self.steps.last:
            form = self.get_form(
                step=self.steps.first,
                data=self.storage.get_step_data(self.steps.first),
            )
            form.full_clean()

            kwargs['submissions'] = form.get_queryset()
        return kwargs

    def render_next_step(self, form, **kwargs):
        next_step = self.steps.next

        if next_step == self.steps.last and not form.get_queryset().exists():
            self.storage.reset()
            self.admin.message_user(self.request, gettext("No records found"), level=messages.WARNING)
            export_url = 'admin:{}'.format(self.admin.get_admin_url('export'))
            return redirect(export_url)
        return super(FormExportWizardView, self).render_next_step(form, **kwargs)

    def get_content_type(self):
        content_type = mimetype_map.get(
            self.file_type,
            'application/octet-stream'
        )
        return content_type

    def done(self, form_list, **kwargs):
        """
        this step only runs if all forms are valid.
        """
        form_iter = iter(form_list)
        step_1_form = next(form_iter)
        step_2_form = next(form_iter)

        fields = step_2_form.get_fields()
        queryset = step_1_form.get_queryset()

        dataset = Exporter(queryset=queryset).get_dataset(fields=fields)
        filename = step_1_form.get_filename(extension=self.file_type)

        response = HttpResponse(getattr(dataset, self.file_type), content_type=self.get_content_type())
        response['Content-Disposition'] = 'attachment; filename=%s' % filename
        return response



from django.conf import settings
from django.contrib import admin
from django.template.loader import render_to_string

from tablib import Dataset

from ..models import FormSubmission, Webhook
from .base import BaseFormSubmissionAdmin
from .forms import WebhookAdminForm
from .views import FormExportWizardView


def get_supported_format():
    """Get supported format from types xlsx, xls, tsv or cvs."""
    dataset = Dataset()
    for ext in ('xlsx', 'xls'):
        try:
            getattr(dataset, ext)
            return ext
        except (ImportError, AttributeError):
            pass
    return 'csv'


def display_form_submission_data(instance: FormSubmission) -> str:
    context = {"data": instance.get_form_data()}
    return render_to_string("admin/aldryn_forms/display/submission_display_fields.html", context)


class FormSubmissionAdmin(BaseFormSubmissionAdmin):
    readonly_fields = BaseFormSubmissionAdmin.readonly_fields + ['form_url']

    def get_form_export_view(self):
        return FormExportWizardView.as_view(admin=self, file_type=get_supported_format())


class WebhookAdmin(admin.ModelAdmin):
    form = WebhookAdminForm

    def changeform_view(self, request, object_id=None, form_url="", extra_context=None):
        if extra_context is None:
            extra_context = {}
        extra_context["SITE_API_ROOT"] = getattr(settings, "SITE_API_ROOT", "")
        return super().changeform_view(request, object_id, form_url, extra_context)


admin.site.register(FormSubmission, FormSubmissionAdmin)
admin.site.register(Webhook, WebhookAdmin)
